#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import gzip
import StringIO


def smart_compress_gzip(data):
    ret = data
    if len(data) >= 1024:
        buf = StringIO.StringIO()
        f = gzip.GzipFile(fileobj=buf, mode='wb')
        f.write(data)
        f.flush()
        f.close()
        zip_data = buf.getvalue()
        buf.close()
        if len(zip_data) < len(data):
            ret = zip_data
    return ret


def smart_uncompress_gzip(data):
    ret = data
    if len(data) >= 2 and ord(data[0]) == 0x1F and ord(data[1]) == 0x8B:
        buf = StringIO.StringIO(data)
        f = gzip.GzipFile(fileobj=buf)
        ret = f.read()
        f.close()
    return ret


if __name__ == '__main__':
    data = 'abc'
    s = smart_uncompress_gzip(smart_compress_gzip(data))
    if data == s:
        print 'test ok: %s' % (s)
    else:
        print 'test failed: %s' % (s)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import time
import vendor.tqdm as tqdm

_stdout = sys.stdout
_stderr = sys.stderr


class DummyStdout(object):
    """Dummy file-like that will write to tqdm"""
    def __init__(self, file):
        self.file = file

    def write(self, line):
        line = line.rstrip()
        if len(line) > 0:
            tqdm.tqdm.write(line, file=self.file)

    def flush(self):
        self.file.flush()
        # return getattr(self.file, "flush", lambda: None)()


class ProgressBar(object):
    """
    Progress bar"""
    def __init__(self, total):
        self.total = total
        self.value = 0
        # 46%|████     | 116/250 [Time: 00:04, ETA: 00:05, 24.93it/s]
        fmt = '{desc}{percentage:3.0f}%|{bar}| {n_fmt}/{total_fmt} [Time: {elapsed}, ETA: {remaining}, {rate_fmt}]{postfix}'
        self.bar = tqdm.tqdm(total=total, miniters=0, unit='task', bar_format=fmt)

    def start(self):
        # Redirect stdout and stderr to tqdm.write()
        sys.stdout = DummyStdout(_stdout)
        sys.stderr = DummyStdout(_stderr)
        self.update(0)

    def close(self):
        self.bar.close()
        sys.stdout = _stdout
        sys.stderr = _stderr

    def finish(self):
        v = self.total - self.value
        if v > 0:
            self.bar.update(v)

    def set_msg(self, msg, refresh=True):
        self.bar.set_postfix_str(msg, refresh)

    def update(self, value):
        if value < 0:
            value = 0
        elif value > self.total:
            value = self.total
        v = value - self.value
        if v >= 0:
            self.bar.update(v)
            self.value = value


if __name__ == '__main__':
    # 97%|█████████▋| 97/100 [00:01<00:00, 49.70it/s]
    # 100%|██████████| 100/100 [00:02<00:00, 49.58it/s]
    bar = ProgressBar(250)
    bar.start()
    for i in range(bar.total):
        time.sleep(10.0 / bar.total)
        status = ''
        if i < 10:
            status = 'preparing ...'
        elif i < 20:
            status = 'committing ...'
        elif i < 50:
            status = 'pending(%s tasks ahead) ...' % (i)
        elif i < 90:
            status = 'executing ...'
        else:
            status = 'finished'
        bar.set_msg(status)
        bar.update(i)
    bar.finish()
    bar.close()
    print 'over'



from nose.plugins.skip import SkipTest

from tqdm import tqdm
from tests_tqdm import with_setup, pretest, posttest, StringIO, closing


@with_setup(pretest, posttest)
def test_pandas_groupby_apply():
    """Test pandas.DataFrame.groupby(...).progress_apply"""
    try:
        from numpy.random import randint
        import pandas as pd
    except:
        raise SkipTest

    with closing(StringIO()) as our_file:
        tqdm.pandas(file=our_file, leave=False, ascii=True)

        df = pd.DataFrame(randint(0, 50, (500, 3)))
        df.groupby(0).progress_apply(lambda x: None)

        dfs = pd.DataFrame(randint(0, 50, (500, 3)), columns=list('abc'))
        dfs.groupby(['a']).progress_apply(lambda x: None)

        our_file.seek(0)

        # don't expect final output since no `leave` and
        # high dynamic `miniters`
        nexres = '100%|##########|'
        if nexres in our_file.read():
            our_file.seek(0)
            raise AssertionError("\nDid not expect:\n{0}\nIn:{1}\n".format(
                nexres, our_file.read()))


@with_setup(pretest, posttest)
def test_pandas_apply():
    """Test pandas.DataFrame[.series].progress_apply"""
    try:
        from numpy.random import randint
        import pandas as pd
    except:
        raise SkipTest

    with closing(StringIO()) as our_file:
        tqdm.pandas(file=our_file, leave=True, ascii=True)
        df = pd.DataFrame(randint(0, 50, (500, 3)))
        df.progress_apply(lambda x: None)

        dfs = pd.DataFrame(randint(0, 50, (500, 3)), columns=list('abc'))
        dfs.a.progress_apply(lambda x: None)

        our_file.seek(0)

        if our_file.read().count('100%') < 2:
            our_file.seek(0)
            raise AssertionError("\nExpected:\n{0}\nIn:{1}\n".format(
                '100% at least twice', our_file.read()))


@with_setup(pretest, posttest)
def test_pandas_map():
    """Test pandas.Series.progress_map"""
    try:
        from numpy.random import randint
        import pandas as pd
    except:
        raise SkipTest

    with closing(StringIO()) as our_file:
        tqdm.pandas(file=our_file, leave=True, ascii=True)
        dfs = pd.DataFrame(randint(0, 50, (500, 3)), columns=list('abc'))
        dfs.a.progress_map(lambda x: None)

        if our_file.getvalue().count('100%') < 1:
            raise AssertionError("\nExpected:\n{0}\nIn:{1}\n".format(
                '100% at least twice', our_file.getvalue()))


@with_setup(pretest, posttest)
def test_pandas_leave():
    """Test pandas with `leave=True`"""
    try:
        from numpy.random import randint
        import pandas as pd
    except:
        raise SkipTest

    with closing(StringIO()) as our_file:
        df = pd.DataFrame(randint(0, 100, (1000, 6)))
        tqdm.pandas(file=our_file, leave=True, ascii=True)
        df.groupby(0).progress_apply(lambda x: None)

        our_file.seek(0)

        exres = '100%|##########| 101/101'
        if exres not in our_file.read():
            our_file.seek(0)
            raise AssertionError(
                "\nExpected:\n{0}\nIn:{1}\n".format(exres, our_file.read()))


@with_setup(pretest, posttest)
def test_pandas_deprecation():
    """Test bar object instance as argument deprecation"""
    try:
        from numpy.random import randint
        from tqdm import tqdm_pandas
        import pandas as pd
    except:
        raise SkipTest

    with closing(StringIO()) as our_file:
        tqdm_pandas(tqdm(file=our_file, leave=False, ascii=True, ncols=20))
        df = pd.DataFrame(randint(0, 50, (500, 3)))
        df.groupby(0).progress_apply(lambda x: None)
        # Check deprecation message
        assert "TqdmDeprecationWarning" in our_file.getvalue()
        assert "instead of `tqdm_pandas(tqdm(...))`" in our_file.getvalue()

    with closing(StringIO()) as our_file:
        tqdm_pandas(tqdm, file=our_file, leave=False, ascii=True, ncols=20)
        df = pd.DataFrame(randint(0, 50, (500, 3)))
        df.groupby(0).progress_apply(lambda x: None)
        # Check deprecation message
        assert "TqdmDeprecationWarning" in our_file.getvalue()
        assert "instead of `tqdm_pandas(tqdm, ...)`" in our_file.getvalue()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import time
import re
import hashlib
import httplib
import urllib
import urllib2
import socket
import StringIO
import json
import gzip
import serialize
import define
import utils
from CoralResultSet import CoralResultSet
from CoralParallelJob import CoralParallelJob

class CoralDBClient(object):
    """
    CoralDB Client
    Usage:
    >>> client = CoralDBClient("coraldb://127.0.0.1:80")
    >>> client.login("test", "123456")
    """
    def __init__(self, address):
        # url = 'coraldb://test:123456@localhost:8086'
        p = re.compile(r"""^coraldb://(([^:]*)?(:[^@]*)?@)?([^:]+):(\d+)$""", re.IGNORECASE)
        m = p.match(address)
        if not m:
            raise Exception('illegal CoralDB address: ' + address)
        self.username = m.group(2)
        self.password = m.group(3)[1:] if m.group(3) else None
        if self.password:
            md5 = hashlib.md5()
            md5.update(self.password)
            self.password = md5.hexdigest()
        self._host = m.group(4)
        self._port = int(m.group(5))
        self._token = None
        self.timeout = 60
        self.compress = False

    def login(self, username=None, password=None):
        """
        login: 登陆
        :param username: 用户名
        :param password: 密码
        :return: None
        :exception: 登陆失败抛出异常
        """
        if username is not None:
            self.username = username
        if password is not None:
            self.password = utils.encodePwd(password)
        params = {'u': self.username, 'p': self.password}
        rep = self.__request('login', params, login=True)
        if rep['error'] == "":
            self._token = rep['token']
        return CoralResultSet()

    def changePwd(self, password):
        """
        changePwd: 修改密码
        :param password: 新密码
        :return: None
        :exception 失败报出异常
        """
        if password is None or password == "":
            raise Exception('new password can not be empty')
        params = {'p': self.password, 'newPwd': password, 'token': self._token}
        self.__request('changePwd', params)
        return CoralResultSet()

    def getTradingDays(self, beginDate=0, endDate=0, market=define.SH):
        """
        getTradingDays: 获取交易日序列
        :param beginDate: 开始日期，默认为系统支持的最早交易日，示例：20161205
        :param endDate: 结束日期，默认为系统支持的最晚交易日，示例：20161206
        :param market: 市场，默认为上海
        :return: 交易日列表
        """
        params = {'beginDate': beginDate, 'endDate': endDate, 'market': market}
        rep = self.__request('getTradingDays', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getStat(self, beginDate=0, endDate=0):
        """
        getStat: 获取数据统计信息
        :param beginDate: 开始日期，默认为系统中存在数据的最早日期，示例：20161205
        :param endDate: 结束日期，默认为系统中存在数据的最晚日期，示例：20161206
        :return: 结果集
        """
        params = {'beginDate': beginDate, 'endDate': endDate}
        rep = self.__request('getStat', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getCode(self, sector='', date=0, fields="*"):
        """
        getCode: 获取系统内置板块信息
        :param sector: 板块名称，多个名称用逗号分隔，默认获取所有板块
        :param date: 数据日期，默认为每只证券最新日期的数据，示例：20161205
        :param fields: 字段名，多个字段逗号分隔，默认为所有字段
        :return: 结果集
        """
        params = {'sector': sector, 'date': date, 'fields': fields}
        rep = self.__request('getCode', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getBar(self, code=[], beginDate=0, endDate=0, cycle=0, fields="*", beginTime=0, endTime=0, holo=False, holoInterval=0):
        """
        getBar: 获取行情数据
        :param code: 证券代码或证券代码列表
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param cycle: 周期，默认为原始数据
        :param fields: 字段名，多个字段逗号分隔，默认为所有字段
        :param beginTime: 开始时间，默认为当天开始，示例：93000
        :param endTime: 结束时间，默认为当天结束，示例：150000
        :param holo: 是否查全息快照表，默认查Level2快照表
        :param holoInterval: 全息快照时间间隔，默认为1000毫秒
        :return: 结果集
        """
        if not code:
            raise Exception("code cannot be empty")
        code = ','.join(code if isinstance(code, list) else [code])
        params = {'code': code, 'beginDate': beginDate, 'endDate': endDate, 'cycle': cycle, 'fields': fields,
                  'beginTime': beginTime, 'endTime': endTime, 'holo': holo, 'holoInterval': holoInterval}
        rep = self.__request('getBar', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getOrder(self, code=[], beginDate=0, endDate=0, fields="*", beginTime=0, endTime=0):
        """
        getOrder: 获取逐笔委托数据
        :param code: 证券代码或证券代码列表
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fields: 字段名，多个字段逗号分隔，默认为所有字段
        :param beginTime: 开始时间，默认为当天开始，示例：93000
        :param endTime: 结束时间，默认为当天结束，示例：150000
        :return: 结果集
        """
        if not code:
            raise Exception("code cannot be empty")
        code = ','.join(code if isinstance(code, list) else [code])
        params = {'code': code, 'beginDate': beginDate, 'endDate': endDate, 'fields': fields,
                  'beginTime': beginTime, 'endTime': endTime}
        rep = self.__request('getOrder', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getKnock(self, code=[], beginDate=0, endDate=0, fields="*", beginTime=0, endTime=0):
        """
        getOrder: 获取逐笔委托数据
        :param code: 证券代码或证券代码列表
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fields: 字段名，多个字段逗号分隔，默认为所有字段
        :param beginTime: 开始时间，默认为当天开始，示例：93000
        :param endTime: 结束时间，默认为当天结束，示例：150000
        :return: 结果集
        """
        if not code:
            raise Exception("code cannot be empty")
        code = ','.join(code if isinstance(code, list) else [code])
        params = {'code': code, 'beginDate': beginDate, 'endDate': endDate, 'fields': fields,
                  'beginTime': beginTime, 'endTime': endTime}
        rep = self.__request('getKnock', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserData(self, table, query="", sort="", fields="*", offset=0, limit=0, password=""):
        """
        getUserData: 查询用户数据
        :param table: 数据表名
        :param query: 查询条件，示例：{"name": "foo", "age": {"$gt": 18}}
        :param sort: 排序条件，示例：name,-age
        :param fields: 字段名，多个字段逗号分隔，默认为所有字段
        :param offset: 偏移量，默认为0
        :param limit: 查询结果数量限制，默认无限制
        :param password: 访问密码，访问其他人的数据时需要填写
        :return: 结果集
        """
        if not table:
            raise Exception("table is required")
        query = json.dumps(query) if query else ""
        if password:
            password = utils.encodePwd(password)
        params = {'table': table, 'query': query, 'sort': sort, 'fields': fields, 'offset': offset, 'limit': limit, 'password': password}
        ret = []
        if limit <= 0:
            _offset = offset
            _limit = 10000
            while True:
                params['offset'] = _offset
                params['limit'] = _limit
                rep = self.__request('getUserData', params)
                rows = rep['data']
                ret.extend(rows)
                _offset += _limit
                if len(rows) < _limit:
                    break
                if len(ret) >= 100000:
                    raise Exception('result is too large, more then 100,000 records')
        else:
            rep = self.__request('getUserData', params)
            ret = rep['data']
        return ret

    def listUserData(self, username=""):
        """
        listUserData: 查看用户数据信息
        :param username: 用户名，默认为当前用户
        :return:
        """
        params = {'username': username}
        rep = self.__request('listUserData', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def setUserData(self, table, query="", data=[], password=""):
        """
        setUserData: 保存或更新用户数据
        :param table: 数据表名
        :param query: 查询条件，示例：{"name": "foo", "age": {"$gt": 18}}
        :param data: 待保存的数据或列表
        :param password: 访问密码，访问其他人的数据时需要填写
        :return:
        """
        if not table:
            raise Exception("table is required")
        if not data:
            raise Exception("data is required")
        if query:
            if isinstance(data, list):
                raise Exception('data cannot be a list when query is not empty')
        else:
            data = data if isinstance(data, list) else [data]
        query = json.dumps(query) if query else ""
        data = json.dumps(data) if data else ""
        if password:
            password = utils.encodePwd(password)
        body = {'table': table, 'query': query, 'data': data, 'password': password}
        body = json.dumps(body)
        self.__request('setUserData', data=body)

    def delUserData(self, table, query="", password=""):
        """
        delUserData: 删除用户数据
        :param table: 数据表名
        :param query: 查询条件，示例：{"name": "foo", "age": {"$gt": 18}}
        :param password: 访问密码，访问其他人的数据时需要填写
        :return:
        """
        if not table:
            raise Exception("table is required")
        query = json.dumps(query) if query else ""
        if password:
            password = utils.encodePwd(password)
        params = {'table': table, 'query': query, 'password': password}
        self.__request('delUserData', params)

    def setUserDataIndex(self, table, index=[], password=""):
        """
        setUserDataIndex: 设置用户数据索引
        :param table: 数据表名
        :param index: 索引，为空表示删除所有索引，示例：["date,fund_id", "code", "market"]
        :param password: 访问密码，访问其他人的数据时需要填写
        :return:
        """
        if not table:
            raise Exception("table is required")
        index = ';'.join(index if isinstance(index, list) else [index])
        if password:
            password = utils.encodePwd(password)
        params = {'table': table, 'index': index, 'password': password}
        self.__request('setUserDataIndex', params)

    def setUserDataPassword(self, table, roPassword="", rwPassword=""):
        """
        setUserDataPassword: 设置用户数据访问密码
        :param table: 数据表名
        :param roPassword: 只读访问密码，为空表示禁用
        :param rwPassword: 读写访问密码，为空表示禁用
        :return:
        """
        if not table:
            raise Exception("table is required")
        params = {'table': table, 'roPassword': roPassword, 'rwPassword': rwPassword}
        self.__request('setUserDataPassword', params)

    def getUserSpotAsset(self, beginDate=0, endDate=0, fundId=[], username=[]):
        """
        getUserSpotAsset: 获取用户现货资金
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username}
        rep = self.__request('getUserSpotAsset', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserSpotPosition(self, beginDate=0, endDate=0, fundId=[], username=[], code=[]):
        """
        getUserSpotPosition: 获取用户现货持仓
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param code: 代码，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username, 'code': code}
        rep = self.__request('getUserSpotPosition', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserSpotOrder(self, beginDate=0, endDate=0, fundId=[], username=[], code=[], bsFlag=[], orderNo=[]):
        """
        getUserSpotOrder: 获取用户现货委托
        :param date: 日期，示例：20161205
        :param fund_id: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param code: 代码，默认为所有
        :param bsFlag: 买卖标记（1-买入，2-卖出，3-申购，4-赎回），默认为所有
        :param orderNo: 委托合同号，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        bsFlag = ','.join(bsFlag if isinstance(bsFlag, list) else [bsFlag])
        orderNo = ','.join(orderNo if isinstance(orderNo, list) else [orderNo])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username, 'code': code,
                  'bsFlag': bsFlag, 'orderNo': orderNo}
        rep = self.__request('getUserSpotOrder', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserSpotKnock(self, beginDate=0, endDate=0, fundId=[], username=[], code=[], bsFlag=[], matchType=[], orderNo=[]):
        """
        getUserSpotKnock: 获取用户现货成交
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param code: 代码，默认为所有
        :param bsFlag: 买卖标记（1-买入，2-卖出，3-申购，4-赎回），默认为所有
        :param matchType: 成交类型（1-成交，2-撤单，3-废单），默认为所有
        :param orderNo: 委托合同号，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        bsFlag = ','.join(bsFlag if isinstance(bsFlag, list) else [bsFlag])
        matchType = ','.join(matchType if isinstance(matchType, list) else [matchType])
        orderNo = ','.join(orderNo if isinstance(orderNo, list) else [orderNo])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username, 'code': code,
                  'matchType': matchType, 'orderNo': orderNo}
        rep = self.__request('getUserSpotKnock', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserFutureAsset(self, beginDate=0, endDate=0, fundId=[], username=[]):
        """
        getUserFutureAsset: 获取用户期货资金
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username}
        rep = self.__request('getUserFutureAsset', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserFuturePosition(self, beginDate=0, endDate=0, fundId=[], username=[], hedgeFlag=0, code=[]):
        """
        getUserFuturePosition: 获取用户期货持仓
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param hedgeFlag: 套保标记（1-投机，2-套利，3-套保），默认为所有
        :param code: 代码，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username,
                  'hedgeFlag': hedgeFlag, 'code': code}
        rep = self.__request('getUserFuturePosition', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserFutureOrder(self, beginDate=0, endDate=0, fundId=[], username=[], hedgeFlag=0, code=[], bsFlag=[], orderNo=[]):
        """
        getUserFutureOrder: 获取用户期货委托
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param hedgeFlag: 套保标记（1-投机，2-套利，3-套保），默认为所有
        :param code: 代码，默认为所有
        :param bsFlag: 买卖标记（1-买入，2-卖出，3-申购，4-赎回），默认为所有
        :param orderNo: 委托合同号，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        bsFlag = ','.join(bsFlag if isinstance(bsFlag, list) else [bsFlag])
        orderNo = ','.join(orderNo if isinstance(orderNo, list) else [orderNo])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username,
                  'hedgeFlag': hedgeFlag, 'code': code, 'bsFlag': bsFlag, 'orderNo': orderNo}
        rep = self.__request('getUserFutureOrder', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserFutureKnock(self, beginDate=0, endDate=0, fundId=[], username=[], hedgeFlag=0, code=[], bsFlag=[], matchType=[], orderNo=[]):
        """
        getUserFutureKnock: 获取用户期货成交
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param hedgeFlag: 套保标记（1-投机，2-套利，3-套保），默认为所有
        :param code: 代码，默认为所有
        :param bsFlag: 买卖标记（1-买入，2-卖出，3-申购，4-赎回），默认为所有
        :param matchType: 成交类型（1-成交，2-撤单，3-废单），默认为所有
        :param orderNo: 委托合同号，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        bsFlag = ','.join(bsFlag if isinstance(bsFlag, list) else [bsFlag])
        matchType = ','.join(matchType if isinstance(matchType, list) else [matchType])
        orderNo = ','.join(orderNo if isinstance(orderNo, list) else [orderNo])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username,
                  'hedgeFlag': hedgeFlag, 'code': code, 'bsFlag': bsFlag, 'matchType': matchType, 'orderNo': orderNo}
        rep = self.__request('getUserFutureKnock', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserOptionAsset(self, beginDate=0, endDate=0, fundId=[], username=[]):
        """
        getUserOptionAsset: 获取用户期权资金
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username}
        rep = self.__request('getUserOptionAsset', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserOptionPosition(self, beginDate=0, endDate=0, fundId=[], username=[], code=[]):
        """
        getUserOptionPosition: 获取用户期权持仓
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param code: 代码，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username, 'code': code}
        rep = self.__request('getUserOptionPosition', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserOptionOrder(self, beginDate=0, endDate=0, fundId=[], username=[], code=[], bsFlag=[], orderNo=[]):
        """
        getUserOptionOrder: 获取用户期权委托
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param code: 代码，默认为所有
        :param bsFlag: 买卖标记（1-买入，2-卖出，3-申购，4-赎回），默认为所有
        :param orderNo: 委托合同号，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        bsFlag = ','.join(bsFlag if isinstance(bsFlag, list) else [bsFlag])
        orderNo = ','.join(orderNo if isinstance(orderNo, list) else [orderNo])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username,
                  'code': code, 'bsFlag': bsFlag, 'orderNo': orderNo}
        rep = self.__request('getUserOptionOrder', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getUserOptionKnock(self, beginDate=0, endDate=0, fundId=[], username=[], code=[], bsFlag=[], matchType=[], orderNo=[]):
        """
        getUserOptionKnock: 获取用户期权成交
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fundId: 资金账号，默认为所有
        :param username: 用户名，默认为所有
        :param code: 代码，默认为所有
        :param bsFlag: 买卖标记（1-买入，2-卖出，3-申购，4-赎回），默认为所有
        :param matchType: 成交类型（1-成交，2-撤单，3-废单），默认为所有
        :param orderNo: 委托合同号，默认为所有
        :return:
        """
        fundId = ','.join(fundId if isinstance(fundId, list) else [fundId])
        username = ','.join(username if isinstance(username, list) else [username])
        code = ','.join(code if isinstance(code, list) else [code])
        bsFlag = ','.join(bsFlag if isinstance(bsFlag, list) else [bsFlag])
        matchType = ','.join(matchType if isinstance(matchType, list) else [matchType])
        orderNo = ','.join(orderNo if isinstance(orderNo, list) else [orderNo])
        params = {'beginDate': beginDate, 'endDate': endDate, 'fundId': fundId, 'username': username,
                  'code': code, 'bsFlag': bsFlag, 'matchType': matchType, 'orderNo': orderNo}
        rep = self.__request('getUserOptionKnock', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getNotice(self, code=[], beginDate=0, endDate=0, fields="*", keywords=""):
        """
        getNotice: 获取股票公告
        :param code: 证券代码或证券代码列表
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fields: 字段名，多个字段逗号分隔，默认为所有字段
        :param keywords: 关键词，多个关键词用空格分割
        :return: 
        """
        code = ','.join(code if isinstance(code, list) else [code])
        params = {'beginDate': beginDate, 'endDate': endDate, 'code': code, 'fields': fields, 'keywords': keywords}
        rep = self.__request('getNotice', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def getOptionBaseInfo(self, code=[], beginDate=0, endDate=0, fields="*"):
        """
        getOptionBaseInfo: 获取期权基础信息
        :param code: 证券代码或证券代码列表
        :param beginDate: 开始日期，默认为今天，示例：20161205
        :param endDate: 结束日期，默认等于开始日期，示例：20161206
        :param fields: 字段名，多个字段逗号分隔，默认为所有字段
        :return: 
        """
        code = ','.join(code if isinstance(code, list) else [code])
        params = {'beginDate': beginDate, 'endDate': endDate, 'code': code, 'fields': fields}
        rep = self.__request('getOptionBaseInfo', params)
        return CoralResultSet(rep['columns'], rep['values'])

    def callTs(self, script):
        """
        调用天软TSL脚本
        :param script: 脚本字符串，示例：return getbk("A股");
        :return: 函数返回值
        """
        script = script.strip()
        req = {'func': '', 'args': '', 'script': script}
        data = json.dumps(req)
        rep = self.__request('callTs', data=data)
        result = rep['result']
        return serialize.loads(result)

    def callTsFunc(self, func, args=None):
        """
        调用天软函数
        :param func: 函数名，示例：getbk
        :param args: 函数参数，示例：u'A股'
        :return: 函数返回值
        """
        args = serialize.dumps(args)
        req = {'func': func, 'args': args, 'script': ''}
        data = json.dumps(req)
        rep = self.__request('callTs', data=data)
        result = rep['result']
        return serialize.loads(result)

    def map(self, func, args, context=None, reduce=None):
        """
        :param func: Map函数
        :param args: Map参数
        :param context: 上下文参数
        :param reduce: Reduce函数
        :return: 执行结果
        """
        url = "ws://%s:%s/parallel/run?token=%s" % (self._host, self._port, self._token)
        job = CoralParallelJob(url)
        job.map(func, args, context)
        if reduce:
            job.reduce(reduce)
        job.commit()
        job.wait()
        return job.get()

    def __request(self, path, params=None, data=None, login=False):
        """
        """
        if params is None:
            params = {}
        if self._token:
            params['token'] = self._token
        for k, v in params.iteritems():
            if isinstance(v, unicode):
                params[k] = v.encode('UTF-8')
        url = 'http://%s:%s/%s?%s' % (self._host, self._port, path, urllib.urlencode(params))
        headers = {'User-Agent': 'PyCoralDB/1.1'}
        if data:
            headers['Content-Type'] = 'application/json'
        if self.compress:
            headers['Accept-Encoding'] = 'gzip'
        # Try to send the request a maximum of three times.
        for i in xrange(0, 3):
            try:
                req = urllib2.Request(url, data=data, headers=headers)
                f = urllib2.urlopen(req, timeout=10 if login else self.timeout)
                html = f.read()
                encoding = f.info().get('Content-Encoding')
                if encoding == 'gzip':
                    buf = StringIO.StringIO(html)
                    zf = gzip.GzipFile(fileobj=buf)
                    html = zf.read()
                    zf.close()
                rep = json.loads(html)
                if rep['error']:
                    err = '%s failed: %s' % (path, rep['error'])
                    raise Exception(err.encode('UTF-8'))
                return rep
            except urllib2.URLError, e:
                if not login and e.reason == 'Unauthorized':
                    self.login()
                    if self._token:
                        params['token'] = self._token
                    url = 'http://%s:%s/%s?%s' % (self._host, self._port, path, urllib.urlencode(params))
                    login = True
                    continue
                elif i < 2 and e.reason in ['Too Many Requests', 'Bad Gateway']:
                    time.sleep(10)
                    continue
                elif i < 2 and e.reason not in ['Forbidden', 'Bad Request']:
                    time.sleep(5)
                    continue
                else:
                    raise e
            except socket.timeout, e:
                if i >= 2:
                    raise e
            except socket.error, e:
                time.sleep(5)
                if i >= 2:
                    raise e
            except httplib.UnknownProtocol, e:
                time.sleep(5)
                if i >= 2:
                    raise e
        return None

if __name__ == '__main__':
    client = CoralDBClient('coraldb://127.0.0.1:5166')
    client.timeout = 5
    client.login("test", "123456")
    print client.getTradingDays()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from version import __version__

from define import SH, SZ, CFFEX, SHFE, DEC, CZCE, INE
from define import MS, S, M, H, D
from define import STOCK, FUTURE, OPTION, INDEX
from utils import encodePwd
from CoralDBClient import CoralDBClient
# from CoralParallelClient import CoralParallelClient

from times import rawDate, rawDateText, rawTime, rawTimeText, addRawTime, subRawTime

from serialize import register_main_globals, register_custom_modules



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import hashlib


def encodePwd(plainPwd):
    """
    加密明文密码得到加密后的密码，可用于保存到配置文件中，防止明文密码泄露。
    :param plainPwd: 明文密码
    :return: 加密后的密码
    """
    md5 = hashlib.md5()
    md5.update(plainPwd)
    cypherPwd = md5.hexdigest()
    return cypherPwd



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import time
import datetime
import json
import threading
import websocket
import serialize
import compress
import progressbar


class CoralParallelJob(object):
    """
    Coral Parallel Job
    Apply function to every item of iterable mapper_args and return a list of the results.
    """
    def __init__(self, url):
        """
        :param url: 服务器端地址
        """
        self.url = url
        self.language = 'python'
        self.__map_func = None
        self.__map_args = None
        self.__reduce_func = None
        self.__context = None
        # ----------------------------------
        self.__error = None
        self.__map_results = {} # index -> result
        self.__reduce_results = {}  # index -> result
        self.__sock = websocket.WebSocket()
        self.__thread = threading.Thread(target=self.__run)
        self.__thread.setDaemon(True)
        self.__time = None
        self.__status = ''
        self.__abort = False
        self.__done = False

    def map(self, func, args, context=None):
        self.__map_func = func
        self.__map_args = args
        self.__context = context
        return self

    def reduce(self, func):
        self.__reduce_func = func
        return self

    def commit(self):
        if self.__time is not None:
            raise Exception('job is already committed')
        self.__time = datetime.datetime.now()
        self.__thread.start()
        return self

    def __encode(self):
        if not self.__map_func:
            raise Exception('encode job failed: map_func is required')
        if not self.__map_args:
            raise Exception('encode job failed: map_args is required')
        d = {'language': self.language,
             'map_func': serialize.dumps(self.__map_func),
             'map_args': [serialize.dumps(args) for args in self.__map_args],
             'reduce_func': serialize.dumps(self.__reduce_func) if self.__reduce_func else None,
             'context': serialize.dumps(self.__context) if self.__context else None}
        data = json.dumps(d)
        data = compress.smart_compress_gzip(data)
        return data

    def __run(self):
        try:
            self.__status = 'Connecting ...'
            self.__sock.connect(self.url)
            self.__status = 'Preparing ...'
            req = self.__encode()
            self.__status = 'Committing ...'
            self.__sock.send(req)
        except BaseException, e:
            self.__error = 'error: %s' % e
            self.__done = True
            self.__status = 'Failed'
        while not self.__abort and not self.__done:
            try:
                frame = self.__sock.recv()
                if not frame:
                    self.__error = 'connection is broken'
                    self.__done = True
                    break
                frame = compress.smart_uncompress_gzip(frame)
                task = json.loads(frame)
                type = task['type']
                if type == 3:
                    self.__status = task['status']
                    continue
                elif type == 4:
                    stdType = task['std_type']
                    output = task['output']
                    if stdType == 2:
                        sys.stdout.write(output)
                    else:
                        sys.stderr.write(output)
                    continue
                index = task['index']
                error = task['error']
                output = task['output']
                if output:
                    sys.stdout.write(output)
                if error:
                    pretty_error = error
                    if type == 1:
                        pretty_error = 'map task[%s] error: %s' % (index, error)
                    elif type == 2:
                        pretty_error = 'reduce task error: %s' % error
                    self.__error = pretty_error
                    self.__done = True
                    self.__status = 'Failed'
                    break
                result = serialize.loads(task['result']) if task['result'] else None
                if type == 1:
                    self.__map_results[index] = result
                elif type == 2:
                    self.__reduce_results[index] = result
                if (not self.__reduce_func and len(self.__map_results) >= len(self.__map_args)) or \
                        (self.__reduce_func and len(self.__reduce_results) > 0):
                    self.__done = True
                    self.__status = 'Finished'
                    break
            except BaseException, e:
                self.__error = 'error: %s' % e
                self.__done = True
                self.__status = 'Failed'
                break

    def is_done(self):
        return self.__abort or self.__done

    def wait(self, interactive=True, timeout=None):
        """interactive wait, printing progress at regular intervals"""
        if not interactive:
            self.__thread.join(timeout)
            return
        else:
            if timeout is None:
                timeout = -1
            tic = time.time()
            total = len(self.__map_args) + 1 if self.__reduce_func else len(self.__map_args)
            bar = progressbar.ProgressBar(total=total)
            bar.start()
            while not self.is_done() and (timeout < 0 or time.time() - tic <= timeout):
                self.__thread.join(1.0)
                count = (len(self.__map_results) + len(self.__reduce_results))
                bar.set_msg(self.__status, refresh=False)
                bar.update(count)
            bar.set_msg(self.__status, refresh=True)
            bar.close()

    def abort(self):
        if not self.is_done():
            self.__abort = True
            self.__sock.close()
            self.__error = 'job is aborted'
            self.__status = 'Aborted'

    def get(self):
        if self.__error:
            raise Exception(self.__error)
        ret = None
        if self.__reduce_func:
            if self.__reduce_results:
                ret = self.__reduce_results.values()[0]
        else:
            ret = []
            for k in sorted(self.__map_results.keys()):
                ret.append(self.__map_results[k])
        return ret


if __name__ == '__main__':
    def f(x):
        return x * 2
    token = '505523f5411846a185bb6151e6c5f358'
    url = "ws://127.0.0.1:5167/parallel?token=%s" % token
    job = CoralParallelJob(url)
    job.map(f, range(10)).commit()
    job.wait()
    result = job.get()
    print result


#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os
import base64
import dill as pickle


def loads(b64):
    try:
        return pickle.loads(base64.b64decode(b64))
    except BaseException, e:
        raise Exception('decode obj failed: %s' % e)


def dumps(obj):
    faked = __fake()
    try:
        return base64.b64encode(pickle.dumps(obj, recurse=True))
    except BaseException, e:
        raise Exception('encode obj failed: %s' % e)
    finally:
        __unfake(faked)


# ---------------------------------------------------------------------
# set classes's module in current directory modules to __main__, fake dill to serialize them.

__globals = {}
__modules = None


def register_main_globals(d):
    global __globals
    __globals = d


def register_custom_modules(dir):
    modules = __get_non_main_modules()
    d = __search_modules(dir)
    modules.update(d)


def __get_non_main_modules():
    global __modules
    if __modules is None:
        __modules = {}
        d = __search_modules(os.curdir)
        __modules.update(d)
    return __modules


def __search_modules(base_dir):
    modules = {}
    abs_base_dir = os.path.abspath(base_dir)
    for root, dirs, files in os.walk(abs_base_dir, followlinks=True):
        if abs_base_dir != root and '__init__.py' not in files:
            continue
        for filename in files:
            l_name = filename.lower()
            if not l_name.endswith('.py'):
                continue
            dirname = root[len(abs_base_dir) + 1:]
            s = dirname if l_name == '__init__.py' else os.path.join(dirname, filename[:-3])
            s = s.replace(os.path.sep, '.')
            if s:
                modules[s] = True
    return modules


def __fake():
    modules = __get_non_main_modules()
    faked = []
    for m in __globals.values():
        module_type = '%s' % type(m)
        # if module_type == "<type 'module'>":
        #     filename = getattr(m, '__file__', '')
        #     print 'filename=', filename
        #     if filename:
        #         for dir in __pythonpaths:
        #             if filename.startswith(dir):
        #                 modules[m.__name__] = None
        #                 print 'fake module: [%s]' % (m.__name__)
        #                 break
        if module_type == "<type 'instance'>":
            for module_name in modules:
                if str(m.__class__).startswith(module_name):
                    faked.append((m.__class__, m.__class__.__module__))  # backup
                    m.__class__.__module__ = '__main__'
                    break
    return faked


def __unfake(faked):
    # restore module name is unnecessary
    # for m, v in faked:
    #     m.__module__ = v
    return


if __name__ == '__main__':
    # print __search_modules('.')
    args = [1]
    b64 = dumps(args)
    print b64
    obj = loads(b64)
    print obj


#!/usr/bin/env python
# -*- coding: UTF-8 -*-


class CoralResultSet(object):
    """
    Coral Result Set
    Usage:
    >>> rs.toDataFrame()
    """
    def __init__(self, columns=[], values=[]):
        self.columns = columns if columns is not None else []
        self.values = values if values is not None else []
        
    def items(self):
        return [dict(zip(self.columns, v)) for v in self.values]

    def toDataFrame(self):
        """
        toDataFrame: 转换为pandas.DataFrame对象
        :return: DataFrame对象
        """
        import pandas as pd
        df = pd.DataFrame(self.values, columns=self.columns)
        if self.columns and self.columns[0] == 'timestamp':
            df['timestamp'] = pd.to_datetime(df['timestamp'], format="%Y%m%d%H%M%S%f")
            df = df.set_index(df['timestamp'])
        return df

    def toCSV(self, *args, **kwargs):
        """
        Write ResultSet to a comma-separated values (csv) file
        Example: rs.toCSV('data.csv', encoding='GBK')
        @see Pandas to_csv()
        :param args: 
        :param kwargs: 
        :return: 
        """
        self.toDataFrame().to_csv(*args, **kwargs)
    
    def __len__(self):
        # if isinstance(self.values, numpy.ndarray):
        #     return self.values.shape[0]
        return len(self.values) if self.values else 0
        
    def __iter__(self):
        return CoralResultSetIterator(self)

    def __getitem__(self, index):
        if isinstance(index, slice):
            return [dict(zip(self.columns, row)) for row in self.values[index]]
        else:
            return dict(zip(self.columns, self.values[index]))

    def __setitem__(self, index, value):
        if isinstance(index, slice):
            self.values[index] = [[row.get(name) for name in self.columns] for row in value]
        else:
            self.values[index] = [value.get(name) for name in self.columns]

    def __str__(self, *args, **kwargs):
        return 'CoralResultSet{columns: %s, values: %s}' % (self.columns, self.values)


class CoralResultSetIterator(object):
    def __init__(self, rs):
        self.rs = rs
        self.i = 0
        self.n = len(rs.values)

    def __iter__(self):
        return self

    def next(self):
        if self.i < self.n:
            i = self.i
            self.i += 1
            return dict(zip(self.rs.columns, self.rs.values[i]))
        else:
            raise StopIteration()   


if __name__ == '__main__':
    li = [1, 2, 3]
    rs = CoralResultSet([], [])
    print rs
    rs = CoralResultSet(['code', 'name'], [['60000.SH', u'浦发银行'], ['000001.SZ', u'平安银行']])
    print 'rs0 = %s' % rs[0]
    print 'rs0-1 = %s' % rs[0:200]
    rs[0] = {'code': '600001.SH', 'name': 'name1'}
    print 'rs0 = %s' % rs[0]
    rs[0:10] = [{'code': 'code1', 'name': u'浦发银行1'},  {'code': 'code2', 'name': 'name2'}]
    print 'rs0-1 = %s' % rs[0]
    print '-------------'
    print rs
    # rs.toCSV('D:/a.csv', encoding='GBK')
    # for item in rs:
    #     print item



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import version

NAME = "pycoraldb"
VERSION = version.__version__

install_requires = ['websocket-client>=0.44.0', 'dill>=0.2.7.1']

setup(
    name=NAME,
    version=VERSION,
    packages=['pycoraldb', 'pycoraldb.vendor', 'pycoraldb.vendor.tqdm'],
    package_dir={'': '..'},
    package_data={'': ['*.md', '*.py']},
    install_requires=install_requires,
    # install_requires=['pandas>=2.1'],
    # metadata for upload to PyPI
    author='FanCapital',
    author_email='public@fancapital.com',
    description='Python CoralDB Client',
    long_description='Python CoralDB Client',
    license='FanCapital Public License',
    url='https://pypi.python.org/pypi/pycoraldb',
    keywords='pycoraldb coraldb')



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# 市场代码
SH = 1  # 市场代码：上海
SZ = 2  # 市场代码：深圳
CFFEX = 5  # 市场代码：中金所
SHFE = 6  # 市场代码：上期所
DEC = 7  # 市场代码：郑商所
CZCE = 8  # 市场代码：郑商所
INE = 21  # 市场代码：上海国际能源交易中心

# 周期
MS = 1
S = 1000
M = 60 * S
H = 60 * M
D = 24 * H

# 行情类型
STOCK = 1
FUTURE = 2
OPTION = 3
INDEX = 4



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import time
import re
import hashlib
import httplib
import urllib
import urllib2
import socket
import StringIO
import json
import gzip
import uuid
import base64
import threading
import websocket
import utils
from CoralResultSet import CoralResultSet
from CoralParallelJob import CoralParallelJob


class CoralParallelClient(object):
    """
    Coral Parallel Client
    """
    def __init__(self, address):
        # url = 'coraldb://test:123456@localhost:8086'
        p = re.compile(r"""^coraldb://(([^:]*)?(:[^@]*)?@)?([^:]+):(\d+)$""", re.IGNORECASE)
        m = p.match(address)
        if not m:
            raise Exception('illegal CoralDB address: ' + address)
        self.username = m.group(2)
        self.password = m.group(3)[1:] if m.group(3) else None
        if self.password:
            md5 = hashlib.md5()
            md5.update(self.password)
            self.password = md5.hexdigest()
        self._host = m.group(4)
        self._port = int(m.group(5))
        self._token = None
        self.timeout = 300
        self.compress = False

    def login(self, username=None, password=None):
        """
        login: 登陆
        :param username: 用户名
        :param password: 密码
        :return: None
        :exception: 登陆失败抛出异常
        """
        if username is not None:
            self.username = username
        if password is not None:
            self.password = utils.encodePwd(password)
        params = {'u': self.username, 'p': self.password}
        rep = self.__request('parallel/login', params, login=True)
        if rep['error'] == "":
            self._token = rep['token']
        return CoralResultSet()

    def map(self, func=None, args=None, context=None):
        """
        :param func: Map函数
        :param args: Map参数
        :param context: 上下文参数
        :return: CoralParallelJob对象
        """
        url = "ws://%s:%s//parallel/run?token=%s" % (self._host, self._port, self._token)
        print url
        job = CoralParallelJob(url)
        return job.map(func, args, context)

    def __request(self, path, params=None, data=None, login=False):
        """
        """
        if params is None:
            params = {}
        if self._token:
            params['token'] = self._token
        for k, v in params.iteritems():
            if isinstance(v, unicode):
                params[k] = v.encode('UTF-8')
        url = 'http://%s:%s/%s?%s' % (self._host, self._port, path, urllib.urlencode(params))
        headers = {'User-Agent': 'PyCoralDB/1.1'}
        if self.compress:
            headers['Accept-Encoding'] = 'gzip'
        # Try to send the request a maximum of three times.
        for i in xrange(0, 3):
            try:
                req = urllib2.Request(url, data=data, headers=headers)
                f = urllib2.urlopen(req, timeout=10 if login else self.timeout)
                html = f.read()
                encoding = f.info().get('Content-Encoding')
                if encoding == 'gzip':
                    buf = StringIO.StringIO(html)
                    zf = gzip.GzipFile(fileobj=buf)
                    html = zf.read()
                    zf.close()
                rep = json.loads(html)
                if rep['error'] != "":
                    err = '%s failed: %s' % (path, rep['error'])
                    raise Exception(err.encode('UTF-8'))
                return rep
            except urllib2.URLError, e:
                if not login and e.reason == 'Unauthorized':
                    self.login()
                    if self._token:
                        params['token'] = self._token
                    url = 'http://%s:%s/%s?%s' % (self._host, self._port, path, urllib.urlencode(params))
                    login = True
                    continue
                elif i < 2 and e.reason in ['Too Many Requests', 'Bad Gateway']:
                    time.sleep(10)
                    continue
                elif i < 2 and e.reason not in ['Forbidden', 'Bad Request']:
                    time.sleep(5)
                    continue
                else:
                    raise e
            except socket.timeout, e:
                if i >= 2:
                    raise e
            except socket.error, e:
                time.sleep(5)
                if i >= 2:
                    raise e
            except httplib.UnknownProtocol, e:
                time.sleep(5)
                if i >= 2:
                    raise e
        return None


if __name__ == '__main__':
    client = CoralParallelClient('coraldb://127.0.0.1:5167')
    client.login('test', '123456')
    def f(x):
        return x * 2
    pr = client.map(f, range(5)).commit()
    pr.wait_interactive()
    print pr.get()




#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import logging as log
import datetime

class UTC(datetime.tzinfo):
    """
    Coordinated Universal Time, UTC
    https://www.ietf.org/rfc/rfc3339.txt
    """
    def __init__(self, offset = 0):
        self._offset = offset

    def utcoffset(self, dt):
        return datetime.timedelta(hours=self._offset)

    def tzname(self, dt):
        return self.__str__()
# 
    def dst(self, dt):
        return datetime.timedelta(hours=self._offset)
    
    def __str__(self, *args, **kwargs):
        return "UTC +%02d" % (self._offset) if self._offset >= 0 else "UTC -%02d" % (-self._offset)


def rawDate():
    return int(datetime.datetime.now().strftime('%Y%m%d'))


def rawDateText(rDate=None):
    if rDate is None:
        rDate = int(datetime.datetime.now().strftime('%Y%m%d'))
    return '%04d-%02d-%02d' % (rDate/10000, rDate%10000/100, rDate%100)


def rawTime():
    s = datetime.datetime.now().strftime('%H%M%S%f')
    tm = long(s[:-3])
    return tm


def rawTimeText(rTime=None):
    if rTime is None:
        return datetime.datetime.now().strftime('%H:%M:%S.%f')[:-3]
    else:
        return '%02d:%02d:%02d.%03d' % (rTime/10000000, rTime%10000000/100000, rTime%100000/1000, rTime%1000)

def addRawTime(rTime, ms):
    _ms = (rTime / 10000000) * 3600000 + (rTime % 10000000 / 100000) * 60000 + (rTime % 100000 / 1000) * 1000 + rTime % 1000
    _ms += ms
    return _ms / 3600000 * 10000000 + _ms % 3600000 / 60000 * 100000 + _ms % 60000 / 1000 * 1000 + _ms % 1000


def subRawTime(t1, t2):
    ms1 = (t1 / 10000000) * 3600000 + (t1 % 10000000 / 100000) * 60000 + (t1 % 100000 / 1000) * 1000 + t1 % 1000
    ms2 = (t2 / 10000000) * 3600000 + (t2 % 10000000 / 100000) * 60000 + (t2 % 100000 / 1000) * 1000 + t2 % 1000
    return ms1 - ms2


if __name__ == '__main__':
    pass